/*
 *  
 *  Copyright (C) 2004 Aurelien Alleaume <slts@free.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/* read two u8 in  the eeprom. offset must be even. */

#include "pvrusb2.h"
#include "pvrusb2-tuner.h"

#define PVR_EEPROM_I2C_ADDR    0x50

static int pvr_read_eeprom(struct pvr *pvr, u8 offset, u8 *res)
{
    return pvr_i2c_read(pvr, PVR_EEPROM_I2C_ADDR, &offset, 1, res, 2);
}

/*
 * Parse one tag in eeprom and perform associated action.
 */

static u8 *pvr_eeprom_parse(struct pvr *pvr, u8 *data, int *rem)
{
    int length;

    if (*rem == 0)
        return data;

    if (*data == 0x84) {
        length = data[1] + (data[2] << 8);
        data += 3;
        *rem -= 3;
    } else if ((*data & 0xf0) == 0x70) {
        if ((*data & 0x08)){
            /*This is the end... we might verify checksum */
            *rem = 0;
            return data;
        }
        length = *data & 0x07;
        data++;
        (*rem)--;
    } else {
        pvr_debug("Unexpected eeprom format (0x%02x).", *data);
        *rem = 0;
        return data;
    }

    *rem -= length;

    if (*rem < 0){
        /* Ouch, don't go further */
        pvr_debug("Unexpected eeprom format");
        *rem = 0;
        return data;
    }

    /* now *data contains the tag */
    switch(*data){
        case 0x05:
            pvr_debug("audio processor idx : %i", data[1] & 0x0f);
            break;
        case 0x0a:
            /* pvr_debug("tuner format : %i", data[1]); */
            pvr_debug("tuner idx : %i", data[2]);
            pvr_set_tuner(pvr, data[2]);
            break;
        case 0x0e:
            pvr_debug("radio present : %i", data[1]);
            break;
        default:
            break;
    }
    data += length;

    return data;
}

/*
 * Read and analyse data in the eeprom.
 */

int pvr_eeprom_analyse(struct pvr *pvr)
{
    int ret = 0, i, rem;
    u8 *buff, *data;

    pvr_debug("pvr_analyse_eeprom");

    buff = kmalloc(256, GFP_KERNEL);
    if (buff == NULL){
        /* FIXME : fill with generic data ? */
        return -ENOMEM;
    }

    for (i = 0; (i < 256) && (ret == 0); i+=2)
        ret |= pvr_read_eeprom(pvr, i, &buff[i]);

    if (ret){
        /* FIXME : fill with generic data ? */
        kfree(buff);
        return ret;
    }

    /* tagged data is in the second half. Don't know what's in the first */
    data = buff + 128;
    rem = 128;
    while(rem)
        data = pvr_eeprom_parse(pvr, data, &rem);

    kfree(buff);

    return ret;
}
